"""
    This **module** includes unittests of the basic functions of Xponge.process
"""
import os

__all__ = ["test_impose",
           "test_solvent_process",
           "test_lattice"]

def test_impose():
    """
        test imposing bonds, angles and dihedrals
    """
    import Xponge
    import Xponge.forcefield.amber.ff14sb
    import numpy as np
    globals().update(Xponge.ResidueType.get_all_types())

    t1 = ACE + ALA + NME
    ala = t1.residues[1]
    pi = 3.141592654
    Xponge.impose_bond(t1, ala.C, ala.CA, 1.2)
    Xponge.impose_angle(t1, ala.C, ala.CA, ala.N, pi / 2)
    Xponge.impose_dihedral(t1, ala.O, ala.C, ala.CA, ala.N, -pi)
    c = np.array([ala.C.x, ala.C.y, ala.C.z])
    ca = np.array([ala.CA.x, ala.CA.y, ala.CA.z])
    o = np.array([ala.O.x, ala.O.y, ala.O.z])
    n = np.array([ala.N.x, ala.N.y, ala.N.z])
    assert abs(np.linalg.norm(c - ca) - 1.2) < 0.01
    v1 = c - ca
    v2 = ca - n
    assert abs(np.arccos(np.dot(v1, v2) / np.linalg.norm(v1) / np.linalg.norm(v2)) - pi / 2) < 0.01
    v1, v2, v3 = c - o, ca - c, n - ca
    n1, n2 = np.cross(v1, v2), np.cross(v2, v3)
    k = np.sign(np.dot(n1, v3))
    assert abs(k * np.arccos(np.dot(n1, n2) / np.linalg.norm(n1) / np.linalg.norm(n2)) + pi) < 0.01

def test_solvent_process():
    """
        test the function to add solvents box
    """
    import Xponge
    import Xponge.forcefield.amber.ff14sb
    import Xponge.forcefield.amber.tip3p
    from Xponge.mdrun import run
    globals().update(Xponge.ResidueType.get_all_types())
    t = Xponge.get_peptide_from_sequence("AAAAAAAAAA")
    t = Xponge.add_solvent_box(t, WAT, 20)
    Xponge.main_axis_rotate(t)
    Xponge.solvent_replace(t, WAT, {NA: 5, CL: 5})
    Xponge.h_mass_repartition(t)
    Xponge.save_sponge_input(t, "ALA")
    assert run(f"SPONGE -default_in_file_prefix ALA -mode NPT -thermostat middle_langevin \
-dt 1e-3 -constrain_mode SHAKE -barostat andersen_barostat -cutoff 8 -neighbor_list_skin_permit 1.5 \
-crd ALA.dat -box ALA.box -mdinfo ALA.info -mdout ALA.out -rst ALA \
> {os.devnull} 2> {os.devnull}") == 0

def test_lattice():
    """
        test the lattice building system
    """
    import Xponge
    from Xponge.forcefield.amber import gaff
    from Xponge.mdrun import run
    from io import StringIO
    s = StringIO(r"""data_358
_audit_creation_date              2023-11-14
_audit_creation_method            'Materials Studio'
_symmetry_space_group_name_H-M    'P1'
_symmetry_Int_Tables_number       1
_symmetry_cell_setting            triclinic
loop_
_symmetry_equiv_pos_as_xyz
  x,y,z
_cell_length_a                    32.9466
_cell_length_b                    57.0651
_cell_length_c                    6.2388
_cell_angle_alpha                 90.0000
_cell_angle_beta                  90.0000
_cell_angle_gamma                 90.0000
loop_
_atom_site_label
_atom_site_type_symbol
_atom_site_fract_x
_atom_site_fract_y
_atom_site_fract_z
_atom_site_U_iso_or_equiv
_atom_site_adp_type
_atom_site_occupancy
C1     C     0.45858   0.76381   0.75000   0.00000  Uiso   1.00
C2     C     0.37882   0.79040   0.75000   0.00000  Uiso   1.00
C3     C     0.12106   0.87632   0.75000   0.00000  Uiso   1.00
C4     C     0.04139   0.90288   0.75000   0.00000  Uiso   1.00
C5     C     0.45304   0.78894   0.75000   0.00000  Uiso   1.00
C6     C     0.41619   0.80150   0.75000   0.00000  Uiso   1.00
C7     C     0.30122   0.79415   0.75000   0.00000  Uiso   1.00
C8     C     0.27068   0.81253   0.75000   0.00000  Uiso   1.00
C9     C     0.27170   0.85515   0.75000   0.00000  Uiso   1.00
C10    C     0.22915   0.85422   0.75000   0.00000  Uiso   1.00
C11    C     0.19862   0.87259   0.75000   0.00000  Uiso   1.00
C12    C     0.11906   0.90054   0.75000   0.00000  Uiso   1.00
C13    C     0.08181   0.91266   0.75000   0.00000  Uiso   1.00
C14    C     0.46509   0.70706   0.75000   0.00000  Uiso   1.00
C15    C     0.46467   0.68236   0.75000   0.00000  Uiso   1.00
C16    C     0.53317   0.62855   0.75000   0.00000  Uiso   1.00
C17    C     0.52088   0.60409   0.75000   0.00000  Uiso   1.00
C18    C     0.45644   0.58328   0.75000   0.00000  Uiso   1.00
C19    C     0.47911   0.56248   0.75000   0.00000  Uiso   1.00
C20    C     0.46682   0.53803   0.75000   0.00000  Uiso   1.00
C21    C     0.46467   0.48427   0.75000   0.00000  Uiso   1.00
C22    C     0.46512   0.45959   0.75000   0.00000  Uiso   1.00
C23    C     0.58188   0.75403   0.75000   0.00000  Uiso   1.00
C24    C     0.61914   0.76617   0.75000   0.00000  Uiso   1.00
C25    C     0.66560   0.82733   0.75000   0.00000  Uiso   1.00
C26    C     0.70844   0.83341   0.75000   0.00000  Uiso   1.00
C27    C     0.77187   0.81159   0.75000   0.00000  Uiso   1.00
C28    C     0.79174   0.83333   0.75000   0.00000  Uiso   1.00
C29    C     0.83456   0.83941   0.75000   0.00000  Uiso   1.00
C30    C     0.91627   0.86521   0.75000   0.00000  Uiso   1.00
C31    C     0.95307   0.87778   0.75000   0.00000  Uiso   1.00
O32    O     0.29284   0.77339   0.75000   0.00000  Uiso   1.00
O33    O     0.20700   0.89335   0.75000   0.00000  Uiso   1.00
O34    O     0.56850   0.63474   0.75000   0.00000  Uiso   1.00
O35    O     0.43149   0.53183   0.75000   0.00000  Uiso   1.00
O36    O     0.63866   0.84190   0.75000   0.00000  Uiso   1.00
O37    O     0.86152   0.82484   0.75000   0.00000  Uiso   1.00
N38    N     0.34016   0.80329   0.75000   0.00000  Uiso   1.00
N39    N     0.15970   0.86344   0.75000   0.00000  Uiso   1.00
H40    H     0.47274   0.80244   0.75000   0.00000  Uiso   1.00
H41    H     0.41810   0.82035   0.75000   0.00000  Uiso   1.00
H42    H     0.28811   0.87157   0.75000   0.00000  Uiso   1.00
H43    H     0.14636   0.91094   0.75000   0.00000  Uiso   1.00
H44    H     0.09223   0.92925   0.75000   0.00000  Uiso   1.00
H45    H     0.43498   0.71016   0.75000   0.00000  Uiso   1.00
H46    H     0.43544   0.67388   0.75000   0.00000  Uiso   1.00
H47    H     0.42359   0.58328   0.75000   0.00000  Uiso   1.00
H48    H     0.43542   0.49272   0.75000   0.00000  Uiso   1.00
H49    H     0.93502   0.95650   0.75000   0.00000  Uiso   1.00
H50    H     0.59228   0.73742   0.75000   0.00000  Uiso   1.00
H51    H     0.64647   0.75579   0.75000   0.00000  Uiso   1.00
H52    H     0.78829   0.79517   0.75000   0.00000  Uiso   1.00
H53    H     0.91822   0.84636   0.75000   0.00000  Uiso   1.00
H54    H     0.47276   0.36427   0.75000   0.00000  Uiso   1.00
C55    C     0.95858   0.26381   0.75000   0.00000  Uiso   1.00
C56    C     0.87882   0.29040   0.75000   0.00000  Uiso   1.00
C57    C     0.62106   0.37632   0.75000   0.00000  Uiso   1.00
C58    C     0.54139   0.40288   0.75000   0.00000  Uiso   1.00
C59    C     0.95304   0.28894   0.75000   0.00000  Uiso   1.00
C60    C     0.91619   0.30150   0.75000   0.00000  Uiso   1.00
C61    C     0.80122   0.29415   0.75000   0.00000  Uiso   1.00
C62    C     0.77068   0.31253   0.75000   0.00000  Uiso   1.00
C63    C     0.77170   0.35515   0.75000   0.00000  Uiso   1.00
C64    C     0.72915   0.35422   0.75000   0.00000  Uiso   1.00
C65    C     0.69862   0.37259   0.75000   0.00000  Uiso   1.00
C66    C     0.61906   0.40054   0.75000   0.00000  Uiso   1.00
C67    C     0.58181   0.41266   0.75000   0.00000  Uiso   1.00
C68    C     0.96509   0.20706   0.75000   0.00000  Uiso   1.00
C69    C     0.96467   0.18236   0.75000   0.00000  Uiso   1.00
C70    C     0.03317   0.12855   0.75000   0.00000  Uiso   1.00
C71    C     0.02088   0.10409   0.75000   0.00000  Uiso   1.00
C72    C     0.95644   0.08328   0.75000   0.00000  Uiso   1.00
C73    C     0.97911   0.06248   0.75000   0.00000  Uiso   1.00
C74    C     0.96682   0.03803   0.75000   0.00000  Uiso   1.00
C75    C     0.96467   0.98427   0.75000   0.00000  Uiso   1.00
C76    C     0.96512   0.95959   0.75000   0.00000  Uiso   1.00
C77    C     0.08188   0.25403   0.75000   0.00000  Uiso   1.00
C78    C     0.11914   0.26617   0.75000   0.00000  Uiso   1.00
C79    C     0.16560   0.32733   0.75000   0.00000  Uiso   1.00
C80    C     0.20844   0.33341   0.75000   0.00000  Uiso   1.00
C81    C     0.27187   0.31159   0.75000   0.00000  Uiso   1.00
C82    C     0.29174   0.33333   0.75000   0.00000  Uiso   1.00
C83    C     0.33456   0.33941   0.75000   0.00000  Uiso   1.00
C84    C     0.41627   0.36521   0.75000   0.00000  Uiso   1.00
C85    C     0.45307   0.37778   0.75000   0.00000  Uiso   1.00
O86    O     0.79284   0.27339   0.75000   0.00000  Uiso   1.00
O87    O     0.70700   0.39335   0.75000   0.00000  Uiso   1.00
O88    O     0.06850   0.13474   0.75000   0.00000  Uiso   1.00
O89    O     0.93149   0.03183   0.75000   0.00000  Uiso   1.00
O90    O     0.13866   0.34190   0.75000   0.00000  Uiso   1.00
O91    O     0.36152   0.32484   0.75000   0.00000  Uiso   1.00
N92    N     0.84016   0.30329   0.75000   0.00000  Uiso   1.00
N93    N     0.65970   0.36344   0.75000   0.00000  Uiso   1.00
H94    H     0.97274   0.30244   0.75000   0.00000  Uiso   1.00
H95    H     0.91810   0.32035   0.75000   0.00000  Uiso   1.00
H96    H     0.78811   0.37157   0.75000   0.00000  Uiso   1.00
H97    H     0.64636   0.41094   0.75000   0.00000  Uiso   1.00
H98    H     0.59223   0.42925   0.75000   0.00000  Uiso   1.00
H99    H     0.93498   0.21016   0.75000   0.00000  Uiso   1.00
H100   H     0.93544   0.17388   0.75000   0.00000  Uiso   1.00
H101   H     0.92359   0.08328   0.75000   0.00000  Uiso   1.00
H102   H     0.93542   0.99272   0.75000   0.00000  Uiso   1.00
H103   H     0.43502   0.45650   0.75000   0.00000  Uiso   1.00
H104   H     0.09228   0.23742   0.75000   0.00000  Uiso   1.00
H105   H     0.14647   0.25579   0.75000   0.00000  Uiso   1.00
H106   H     0.28829   0.29517   0.75000   0.00000  Uiso   1.00
H107   H     0.41822   0.34636   0.75000   0.00000  Uiso   1.00
H108   H     0.97276   0.86427   0.75000   0.00000  Uiso   1.00
C217   C     0.54142   0.76381   0.75000   0.00000  Uiso   1.00
C218   C     0.62118   0.79040   0.75000   0.00000  Uiso   1.00
C219   C     0.87894   0.87632   0.75000   0.00000  Uiso   1.00
C220   C     0.95861   0.90288   0.75000   0.00000  Uiso   1.00
C221   C     0.54696   0.78894   0.75000   0.00000  Uiso   1.00
C222   C     0.58381   0.80150   0.75000   0.00000  Uiso   1.00
C223   C     0.69878   0.79415   0.75000   0.00000  Uiso   1.00
C224   C     0.72932   0.81253   0.75000   0.00000  Uiso   1.00
C225   C     0.72830   0.85515   0.75000   0.00000  Uiso   1.00
C226   C     0.77085   0.85422   0.75000   0.00000  Uiso   1.00
C227   C     0.80138   0.87259   0.75000   0.00000  Uiso   1.00
C228   C     0.88094   0.90054   0.75000   0.00000  Uiso   1.00
C229   C     0.91819   0.91266   0.75000   0.00000  Uiso   1.00
C230   C     0.53491   0.70706   0.75000   0.00000  Uiso   1.00
C231   C     0.53533   0.68236   0.75000   0.00000  Uiso   1.00
C232   C     0.46683   0.62855   0.75000   0.00000  Uiso   1.00
C233   C     0.47912   0.60409   0.75000   0.00000  Uiso   1.00
C234   C     0.54356   0.58328   0.75000   0.00000  Uiso   1.00
C235   C     0.52089   0.56248   0.75000   0.00000  Uiso   1.00
C236   C     0.53318   0.53803   0.75000   0.00000  Uiso   1.00
C237   C     0.53533   0.48427   0.75000   0.00000  Uiso   1.00
C238   C     0.53488   0.45959   0.75000   0.00000  Uiso   1.00
C239   C     0.41812   0.75403   0.75000   0.00000  Uiso   1.00
C240   C     0.38086   0.76617   0.75000   0.00000  Uiso   1.00
C241   C     0.33440   0.82733   0.75000   0.00000  Uiso   1.00
C242   C     0.29156   0.83341   0.75000   0.00000  Uiso   1.00
C243   C     0.22813   0.81159   0.75000   0.00000  Uiso   1.00
C244   C     0.20826   0.83333   0.75000   0.00000  Uiso   1.00
C245   C     0.16544   0.83941   0.75000   0.00000  Uiso   1.00
C246   C     0.08373   0.86521   0.75000   0.00000  Uiso   1.00
C247   C     0.04693   0.87778   0.75000   0.00000  Uiso   1.00
O248   O     0.70716   0.77339   0.75000   0.00000  Uiso   1.00
O249   O     0.79300   0.89335   0.75000   0.00000  Uiso   1.00
O250   O     0.43150   0.63474   0.75000   0.00000  Uiso   1.00
O251   O     0.56851   0.53183   0.75000   0.00000  Uiso   1.00
O252   O     0.36134   0.84190   0.75000   0.00000  Uiso   1.00
O253   O     0.13848   0.82484   0.75000   0.00000  Uiso   1.00
N254   N     0.65984   0.80329   0.75000   0.00000  Uiso   1.00
N255   N     0.84030   0.86344   0.75000   0.00000  Uiso   1.00
H256   H     0.52726   0.80244   0.75000   0.00000  Uiso   1.00
H257   H     0.58190   0.82035   0.75000   0.00000  Uiso   1.00
H258   H     0.71189   0.87157   0.75000   0.00000  Uiso   1.00
H259   H     0.85364   0.91094   0.75000   0.00000  Uiso   1.00
H260   H     0.90777   0.92925   0.75000   0.00000  Uiso   1.00
H261   H     0.56502   0.71016   0.75000   0.00000  Uiso   1.00
H262   H     0.56456   0.67388   0.75000   0.00000  Uiso   1.00
H263   H     0.57641   0.58328   0.75000   0.00000  Uiso   1.00
H264   H     0.56458   0.49272   0.75000   0.00000  Uiso   1.00
H265   H     0.06498   0.95650   0.75000   0.00000  Uiso   1.00
H266   H     0.40772   0.73742   0.75000   0.00000  Uiso   1.00
H267   H     0.35353   0.75579   0.75000   0.00000  Uiso   1.00
H268   H     0.21171   0.79517   0.75000   0.00000  Uiso   1.00
H269   H     0.08178   0.84636   0.75000   0.00000  Uiso   1.00
H270   H     0.52724   0.36427   0.75000   0.00000  Uiso   1.00
C271   C     0.04142   0.26381   0.75000   0.00000  Uiso   1.00
C272   C     0.12118   0.29040   0.75000   0.00000  Uiso   1.00
C273   C     0.37894   0.37632   0.75000   0.00000  Uiso   1.00
C274   C     0.45861   0.40288   0.75000   0.00000  Uiso   1.00
C275   C     0.04696   0.28894   0.75000   0.00000  Uiso   1.00
C276   C     0.08381   0.30150   0.75000   0.00000  Uiso   1.00
C277   C     0.19878   0.29415   0.75000   0.00000  Uiso   1.00
C278   C     0.22932   0.31253   0.75000   0.00000  Uiso   1.00
C279   C     0.22830   0.35515   0.75000   0.00000  Uiso   1.00
C280   C     0.27085   0.35422   0.75000   0.00000  Uiso   1.00
C281   C     0.30138   0.37259   0.75000   0.00000  Uiso   1.00
C282   C     0.38094   0.40054   0.75000   0.00000  Uiso   1.00
C283   C     0.41819   0.41266   0.75000   0.00000  Uiso   1.00
C284   C     0.03491   0.20706   0.75000   0.00000  Uiso   1.00
C285   C     0.03533   0.18236   0.75000   0.00000  Uiso   1.00
C286   C     0.96683   0.12855   0.75000   0.00000  Uiso   1.00
C287   C     0.97912   0.10409   0.75000   0.00000  Uiso   1.00
C288   C     0.04356   0.08328   0.75000   0.00000  Uiso   1.00
C289   C     0.02089   0.06248   0.75000   0.00000  Uiso   1.00
C290   C     0.03318   0.03803   0.75000   0.00000  Uiso   1.00
C291   C     0.03533   0.98427   0.75000   0.00000  Uiso   1.00
C292   C     0.03488   0.95959   0.75000   0.00000  Uiso   1.00
C293   C     0.91812   0.25403   0.75000   0.00000  Uiso   1.00
C294   C     0.88086   0.26617   0.75000   0.00000  Uiso   1.00
C295   C     0.83440   0.32733   0.75000   0.00000  Uiso   1.00
C296   C     0.79156   0.33341   0.75000   0.00000  Uiso   1.00
C297   C     0.72813   0.31159   0.75000   0.00000  Uiso   1.00
C298   C     0.70826   0.33333   0.75000   0.00000  Uiso   1.00
C299   C     0.66544   0.33941   0.75000   0.00000  Uiso   1.00
C300   C     0.58373   0.36521   0.75000   0.00000  Uiso   1.00
C301   C     0.54693   0.37778   0.75000   0.00000  Uiso   1.00
O302   O     0.20716   0.27339   0.75000   0.00000  Uiso   1.00
O303   O     0.29300   0.39335   0.75000   0.00000  Uiso   1.00
O304   O     0.93150   0.13474   0.75000   0.00000  Uiso   1.00
O305   O     0.06851   0.03183   0.75000   0.00000  Uiso   1.00
O306   O     0.86134   0.34190   0.75000   0.00000  Uiso   1.00
O307   O     0.63848   0.32484   0.75000   0.00000  Uiso   1.00
N308   N     0.15984   0.30329   0.75000   0.00000  Uiso   1.00
N309   N     0.34030   0.36344   0.75000   0.00000  Uiso   1.00
H310   H     0.02726   0.30244   0.75000   0.00000  Uiso   1.00
H311   H     0.08190   0.32035   0.75000   0.00000  Uiso   1.00
H312   H     0.21189   0.37157   0.75000   0.00000  Uiso   1.00
H313   H     0.35364   0.41094   0.75000   0.00000  Uiso   1.00
H314   H     0.40777   0.42925   0.75000   0.00000  Uiso   1.00
H315   H     0.06502   0.21016   0.75000   0.00000  Uiso   1.00
H316   H     0.06456   0.17388   0.75000   0.00000  Uiso   1.00
H317   H     0.07641   0.08328   0.75000   0.00000  Uiso   1.00
H318   H     0.06458   0.99272   0.75000   0.00000  Uiso   1.00
H319   H     0.56498   0.45650   0.75000   0.00000  Uiso   1.00
H320   H     0.90772   0.23742   0.75000   0.00000  Uiso   1.00
H321   H     0.85353   0.25579   0.75000   0.00000  Uiso   1.00
H322   H     0.71171   0.29517   0.75000   0.00000  Uiso   1.00
H323   H     0.58178   0.34636   0.75000   0.00000  Uiso   1.00
H324   H     0.02724   0.86427   0.75000   0.00000  Uiso   1.00
C433   C     0.50000   0.72240   0.75000   0.00000  Uiso   1.00
C434   C     0.50000   0.66922   0.75000   0.00000  Uiso   1.00
C435   C     0.50000   0.49738   0.75000   0.00000  Uiso   1.00
C436   C     0.50000   0.44427   0.75000   0.00000  Uiso   1.00
N437   N     0.50000   0.64345   0.75000   0.00000  Uiso   1.00
N438   N     0.50000   0.52314   0.75000   0.00000  Uiso   1.00
N439   N     0.50000   0.75001   0.75000   0.00000  Uiso   1.00
C440   C     0.00000   0.22240   0.75000   0.00000  Uiso   1.00
C441   C     0.00000   0.16922   0.75000   0.00000  Uiso   1.00
C442   C     0.00000   0.99738   0.75000   0.00000  Uiso   1.00
C443   C     0.00000   0.94427   0.75000   0.00000  Uiso   1.00
N444   N     0.00000   0.14345   0.75000   0.00000  Uiso   1.00
N445   N     0.00000   0.02314   0.75000   0.00000  Uiso   1.00
N446   N     0.00000   0.25001   0.75000   0.00000  Uiso   1.00
N463   N     0.00000   0.91668   0.75000   0.00000  Uiso   1.00
N464   N     0.50000   0.41668   0.75000   0.00000  Uiso   1.00
loop_
_geom_bond_atom_site_label_1
_geom_bond_atom_site_label_2
_geom_bond_distance
_geom_bond_site_symmetry_2
_ccdc_geom_bond_type
C1     C5      1.446   .     D
C1     C239    1.445   .     S
C1     N439    1.576   .     S
C2     C6      1.385   .     D
C2     N38     1.471   .     S
C2     C240    1.384   .     S
C3     C12     1.384   .     D
C3     N39     1.470   .     S
C3     C246    1.384   .     S
C4     C13     1.444   .     D
C4     C247    1.444   .     S
C4     N463    1.575   .     S
C5     C6      1.410   .     S
C5     H40     1.007   .     S
C6     H41     1.078   .     S
C7     C8      1.453   .     S
C7     O32     1.216   .     D
C7     N38     1.385   .     S
C8     C242    1.376   .     S
C8     C243    1.403   .     D
C9     C10     1.403   .     S
C9     H42     1.082   .     S
C9     C242    1.403   .     D
C10    C11     1.453   .     S
C10    C244    1.377   .     D
C11    O33     1.216   .     D
C11    N39     1.385   .     S
C12    C13     1.409   .     S
C12    H43     1.078   .     S
C13    H44     1.007   .     S
C14    C15     1.410   .     D
C14    H45     1.008   .     S
C14    C433    1.445   .     S
C15    H46     1.078   .     S
C15    C434    1.385   .     S
C16    C17     1.453   .     S
C16    O34     1.216   .     D
C16    N437    1.385   .     S
C17    C233    1.376   .     D
C17    C234    1.403   .     S
C18    C19     1.402   .     D
C18    H47     1.082   .     S
C18    C233    1.403   .     S
C19    C20     1.453   .     S
C19    C235    1.377   .     S
C20    O35     1.217   .     D
C20    N438    1.385   .     S
C21    C22     1.408   .     D
C21    H48     1.078   .     S
C21    C435    1.384   .     S
C22    H103    1.007   .     S
C22    C436    1.444   .     S
C23    C24     1.410   .     S
C23    H50     1.008   .     S
C23    C217    1.445   .     D
C24    H51     1.078   .     S
C24    C218    1.384   .     D
C25    C26     1.453   .     S
C25    O36     1.216   .     D
C25    N254    1.385   .     S
C26    C224    1.376   .     S
C26    C225    1.403   .     D
C27    C28     1.403   .     S
C27    H52     1.082   .     S
C27    C224    1.403   .     D
C28    C29     1.453   .     S
C28    C226    1.377   .     D
C29    O37     1.217   .     D
C29    N255    1.384   .     S
C30    C31     1.409   .     S
C30    H53     1.078   .     S
C30    C219    1.384   .     D
C31    H108    1.008   .     S
C31    C220    1.444   .     D
N38    C241    1.385   .     S
N39    C245    1.384   .     S
H49    C76     1.007   .     S
H54    C85     1.008   .     S
C55    C59     1.446   .     D
C55    C293    1.445   .     S
C55    N446    1.576   1_655 D
C56    C60     1.385   .     D
C56    N92     1.471   .     S
C56    C294    1.384   .     S
C57    C66     1.384   .     D
C57    N93     1.470   .     S
C57    C300    1.384   .     S
C58    C67     1.444   .     D
C58    C301    1.444   .     S
C58    N464    1.575   .     S
C59    C60     1.410   .     S
C59    H94     1.007   .     S
C60    H95     1.078   .     S
C61    C62     1.453   .     S
C61    O86     1.216   .     D
C61    N92     1.385   .     S
C62    C296    1.376   .     S
C62    C297    1.403   .     D
C63    C64     1.403   .     S
C63    H96     1.082   .     S
C63    C296    1.403   .     D
C64    C65     1.453   .     S
C64    C298    1.377   .     D
C65    O87     1.216   .     D
C65    N93     1.385   .     S
C66    C67     1.409   .     S
C66    H97     1.078   .     S
C67    H98     1.007   .     S
C68    C69     1.410   .     D
C68    H99     1.008   .     S
C68    C440    1.445   1_655 S
C69    H100    1.078   .     S
C69    C441    1.385   1_655 S
C70    C71     1.453   .     S
C70    O88     1.216   .     D
C70    N444    1.385   .     S
C71    C288    1.403   .     D
C71    C287    1.376   1_455 D
C72    C73     1.402   .     S
C72    H101    1.082   .     S
C72    C287    1.403   .     D
C73    C74     1.453   .     S
C73    C289    1.377   1_655 S
C74    O89     1.217   .     D
C74    N445    1.385   1_655 S
C75    C76     1.408   .     D
C75    H102    1.078   .     S
C75    C442    1.384   1_655 D
C76    C443    1.444   1_655 S
C77    C78     1.410   .     S
C77    H104    1.008   .     S
C77    C271    1.445   .     D
C78    H105    1.078   .     S
C78    C272    1.384   .     D
C79    C80     1.453   .     S
C79    O90     1.216   .     D
C79    N308    1.385   .     S
C80    C278    1.376   .     S
C80    C279    1.403   .     D
C81    C82     1.403   .     S
C81    H106    1.082   .     S
C81    C278    1.403   .     D
C82    C83     1.453   .     S
C82    C280    1.377   .     D
C83    O91     1.217   .     D
C83    N309    1.384   .     S
C84    C85     1.409   .     S
C84    H107    1.078   .     S
C84    C273    1.384   .     D
C85    C274    1.444   .     D
N92    C295    1.385   .     S
N93    C299    1.384   .     S
C217   C221    1.446   .     S
C217   N439    1.576   .     S
C218   C222    1.385   .     S
C218   N254    1.471   .     S
C219   C228    1.384   .     S
C219   N255    1.470   .     S
C220   C229    1.444   .     S
C220   N463    1.575   1_655 S
C221   C222    1.410   .     D
C221   H256    1.007   .     S
C222   H257    1.078   .     S
C223   C224    1.453   .     S
C223   O248    1.216   .     D
C223   N254    1.385   .     S
C225   C226    1.403   .     S
C225   H258    1.082   .     S
C226   C227    1.453   .     S
C227   O249    1.216   .     D
C227   N255    1.385   .     S
C228   C229    1.409   .     D
C228   H259    1.078   .     S
C229   H260    1.007   .     S
C230   C231    1.410   .     S
C230   H261    1.008   .     S
C230   C433    1.445   .     D
C231   H262    1.078   .     S
C231   C434    1.385   .     D
C232   C233    1.453   .     S
C232   O250    1.216   .     D
C232   N437    1.385   .     S
C234   C235    1.402   .     D
C234   H263    1.082   .     S
C235   C236    1.453   .     S
C236   O251    1.217   .     D
C236   N438    1.385   .     S
C237   C238    1.408   .     S
C237   H264    1.078   .     S
C237   C435    1.384   .     D
C238   H319    1.007   .     S
C238   C436    1.444   .     D
C239   C240    1.410   .     D
C239   H266    1.008   .     S
C240   H267    1.078   .     S
C241   C242    1.453   .     S
C241   O252    1.216   .     D
C243   C244    1.403   .     S
C243   H268    1.082   .     S
C244   C245    1.453   .     S
C245   O253    1.217   .     D
C246   C247    1.409   .     D
C246   H269    1.078   .     S
C247   H324    1.008   .     S
H265   C292    1.007   .     S
H270   C301    1.008   .     S
C271   C275    1.446   .     S
C271   N446    1.576   .     S
C272   C276    1.385   .     S
C272   N308    1.471   .     S
C273   C282    1.384   .     S
C273   N309    1.470   .     S
C274   C283    1.444   .     S
C274   N464    1.575   .     S
C275   C276    1.410   .     D
C275   H310    1.007   .     S
C276   H311    1.078   .     S
C277   C278    1.453   .     S
C277   O302    1.216   .     D
C277   N308    1.385   .     S
C279   C280    1.403   .     S
C279   H312    1.082   .     S
C280   C281    1.453   .     S
C281   O303    1.216   .     D
C281   N309    1.385   .     S
C282   C283    1.409   .     D
C282   H313    1.078   .     S
C283   H314    1.007   .     S
C284   C285    1.410   .     S
C284   H315    1.008   .     S
C284   C440    1.445   .     D
C285   H316    1.078   .     S
C285   C441    1.385   .     D
C286   C287    1.453   .     S
C286   O304    1.216   .     D
C286   N444    1.385   1_655 S
C287   C71     1.376   1_655 D
C288   C289    1.402   .     S
C288   H317    1.082   .     S
C289   C290    1.453   .     S
C289   C73     1.377   1_455 S
C290   O305    1.217   .     D
C290   N445    1.385   .     S
C291   C292    1.408   .     S
C291   H318    1.078   .     S
C291   C442    1.384   .     D
C292   C443    1.444   .     D
C293   C294    1.410   .     D
C293   H320    1.008   .     S
C294   H321    1.078   .     S
C295   C296    1.453   .     S
C295   O306    1.216   .     D
C297   C298    1.403   .     S
C297   H322    1.082   .     S
C298   C299    1.453   .     S
C299   O307    1.217   .     D
C300   C301    1.409   .     D
C300   H323    1.078   .     S
C433   N439    1.576   .     S
C434   N437    1.471   .     S
C435   N438    1.470   .     S
C436   N464    1.574   .     S
C440   N446    1.576   .     S
C440   C68     1.445   1_455 S
C441   N444    1.471   .     S
C441   C69     1.385   1_455 S
C442   C75     1.384   1_455 D
C442   N445    1.470   1_565 S
C443   N463    1.574   .     S
C443   C76     1.444   1_455 S
N444   C286    1.385   1_455 S
N445   C74     1.385   1_455 S
N445   C442    1.470   1_545 S
N446   C55     1.576   1_455 D
N463   C220    1.575   1_455 S""")
    assign, lattice_info = Xponge.get_assignment_from_cif(s)
    assign.determine_atom_type("gaff")
    assign.calculate_charge("tpacm4")
    cof = assign.to_residuetype("cof")
    gaff.parmchk2_gaff(cof, 'lattice.frcmod')
    lattice = Xponge.Lattice(basis_molecule=cof, spacing=[0, 0, 20], **lattice_info)
    box = Xponge.BlockRegion(0, 0, 0, lattice_info["cell_length"][0] + 1, lattice_info["cell_length"][1] + 1, 30)
    mol = lattice.create(box, box)
    Xponge.save_sponge_input(mol, "lattice")
    assert run(f"SPONGE -default_in_file_prefix lattice -mode minimization \
-cutoff 8 -neighbor_list_skin_permit 1.5 \
-crd min.dat -box min.box -mdinfo min.info -mdout min.out -rst min \
> {os.devnull} 2> {os.devnull}") == 0
    assert run(f"SPONGE -default_in_file_prefix lattice -mode NPT -thermostat middle_langevin \
-dt 2e-3 -constrain_mode SHAKE -barostat andersen_barostat -cutoff 8 -neighbor_list_skin_permit 1.5 \
-crd lattice.dat -box lattice.box -mdinfo lattice.info -mdout lattice.out -rst lattice \
-coordinate_in_file min_coordinate.txt > {os.devnull} 2> {os.devnull}") == 0
